#
# slda-sip.R, 24 Jan 22

library("dplyr")
library("lda")
library("psychtm")

# Remove stop words from string
rm_stop_words = function(string, words)
{
stopifnot(is.character(string), is.character(words))
spltted = strsplit(string, " ", fixed = TRUE) # fixed = TRUE for speedup
vapply(spltted, function(x) paste(x[!tolower(x) %in% words], collapse = " "), character(1))
}

tinfo=read.csv("task-info-ascii.csv", as.is=TRUE)

# prep_docs cannot handle NAs
tinfo$AuthorisedByID=NULL
tinfo$DeveloperPerformance=NULL

cl_info=tinfo[!duplicated(tinfo$TaskNumber), ] # Keep data for one developer

no_punct=gsub("[^[:alnum:]///' _]", " ", cl_info$Summary) # remove punctuation
no_punct=gsub(" +", " ", no_punct) # convert multiple spaces to single space

no_stop_words=rm_stop_words(no_punct, tm::stopwords("en"))

cl_info$Summary=no_stop_words

# Summaries containing zero or one word
one_word=which(lengths(gregexpr("[a-zA-Z0-9_]+", cl_info$Summary)) < 2)

cl_info=cl_info[-one_word, ] # remove rows where cleaned Summary contains one word

docs_vocab = prep_docs(cl_info, "Summary")

# gibbs_sldax exits if it encounters documents with less than two 'words'
# So remove such entries.
# still_one=which(rowSums(no_stop_words == 0) >32)
# cl_docs=docs_vocab
# cl_docs$documents=cl_docs$documents[-still_one]

vocab_len = length(docs_vocab$vocab)

set.seed(92850827)
fit_lda = gibbs_sldax(m = 450, burn = 300, thin = 1,
			docs = docs_vocab$documents,
			V = vocab_len,
			K = 10, model = "lda", display_progress = TRUE)

beta_hat = est_beta(fit_lda)
get_coherence(beta_ = beta_hat, docs = docs_vocab$documents)

 
set.seed(44680835)
basic_mod = gibbs_sldax(log(HoursActual) ~ log(HoursEstimate), data = cl_info,
                         m = 450, burn = 300,
                         docs = docs_vocab$documents,
                         V = vocab_len,
                         K = 10, model = "sldax", display_progress = TRUE)
eta_post = post_regression(basic_mod)
summary(eta_post)

waic_all(iter = 5, t(lpd(basic_mod))) # A form of AIC

# Adding ProjectCode results in some LDA topics not being significant
with_projcode = gibbs_sldax(log(HoursActual) ~ log(HoursEstimate)+ProjectCode, data = cl_info,
                         m = 450, burn = 300,
                         docs = docs_vocab$documents,
                         V = vocab_len,
                         K = 10, model = "sldax", display_progress = TRUE)

# get_topwords(est_beta(fit_sldax), 15, docs_vocab$vocab, method = "termscore") %>%
#   print(n = 30)

eta_post = post_regression(with_projcode)
summary(eta_post)

waic_all(iter = 5, t(lpd(with_projcode))) # A form of AIC

# Adding Category results in some LDA topics not being significant
with_category = gibbs_sldax(log(HoursActual) ~ log(HoursEstimate)+Category+ProjectCode, data = cl_info,
                         m = 450, burn = 300,
                         docs = docs_vocab$documents,
                         V = vocab_len,
                         K = 10, model = "sldax", display_progress = TRUE)

# get_topwords(est_beta(fit_sldax), 15, docs_vocab$vocab, method = "termscore") %>%
#   print(n = 30)

eta_post = post_regression(with_category)
summary(eta_post)

waic_all(iter = 5, t(lpd(with_category))) # A form of AIC



